var $ = window.$
$(function () {
  var backendErr
  var backendErrText
  var frontendErr
  var cancel

  var title
  var loginPhone
  var loginPassword
  var loginBtn
  var accessKey // 手机+5位随机数
  var publicKey // 密码加密公钥

  var errTimer = null
  var loginFlag = 0 // 1表示登录中

  // 页面加载完成执行
  $(function () {
    backendErr = $('#back-end-err')
    backendErrText = $('#back-end-err span')
    frontendErr = $('#front-end-err')

    title = $('#title')
    loginPhone = $('#login-phone')
    loginPassword = $('#login-password').children()
    loginBtn = $('#login-btn')
    cancel = $('#cancel')

    eventBind()
    loginPhone.focus() // 光标聚焦
  })

  function eventBind () {
    enterKeyDown()

    loginBtn.click(getPublicKey)
    cancel.click(function () {
      clearTimeout(errTimer)
      backendErr.addClass('hidden')
    })
    $('.item-input').focus(function (e) {
      $(e.target).parent().addClass('inputing')
    }).blur(function (e) {
      $(e.target).parent().removeClass('inputing')
    })
  }

  // 获取加密密码的公钥，成功之后登录
  function getPublicKey () {
    if (loginFlag === 1) return
    if (!checkPhone()) return
    if (!checkPassword()) return
    var phone = loginPhone.val().trim()
    var phoneLen = phone.length
    if (phoneLen > 8) {
      accessKey = phone + randomWord(false, 5)
    } else {
      accessKey = phone + randomWord(false, 14 - phoneLen)
    }
    loginFlag = 1 // 表示正在登陆
    loginBtn.text('登录中...').attr('disabled', 'disabled')
    $.ajax({
      type: 'POST',
      url: 'auth/getPublicKey.do',
      xhrFields: { withCredentials: true },
      data: {
        accessKey: accessKey
      },
      success: function (data) {
        if (data && data.success == true) {
          publicKey = data.publicKey
          submit()
        } else {
          showBackendErr(data.errorMessage)
        }
      },
      complete: function () {
        loginBtn.text('登录').removeAttr('disabled')
        loginFlag = 0
      }
    })
  }

  // 登录提交
  function submit () {
    var account = loginPhone.val().trim()
    var password = loginPassword.val().trim()
    var encrypt = new window.JSEncrypt()
    encrypt.setPublicKey(publicKey)
    var pswEncrypted = encrypt.encrypt(password)
    $.ajax({
      type: 'POST',
      url: 'auth/yzjlogin.do' + window.location.search,
      xhrFields: { withCredentials: true },
      data: {
        systemType: 'mc',
        useraccount: account,
        accessKey: accessKey,
        password: pswEncrypted
      },
      success: function (data) {
        if (data.errorcode != 0) {
          showBackendErr(data.description_cn || data.description)
          loginPhone.focus()
          return
        }
        window.localStorage.setItem('ierp-login-username', data.username || '')
        var userinfo = {
          username: data.username || '',
          avatar: data.avatar || ''
        }
        window.localStorage.setItem('ierp-login-userinfo', JSON.stringify(userinfo))

        window.location.href = data.RedirectURL + window.location.hash || getHash('formId')
      },
      complete: function () {
        loginFlag = 0 // 表示登陆完成
        loginBtn.text('登录').removeAttr('disabled')
      }
    })
  }

  function getHash (name) {
  // 修复safari下请求重定向的时候会把hash值丢失
    var formId = getFormId()
    return formId ? '#/formId=' + formId : ''
  }

  function getFormId () {
    let reg = /formId=(\w*)/
    let r = reg.exec(window.location.href)
    return r && r[1]
  }

  function enterKeyDown () {
    $(document).keydown(function (e) {
      if (e.keyCode === 13) {
        $(e.target).blur()
        getPublicKey()
      }
    })
  }

  // 校验手机号
  function checkPhone () {
    var phone = loginPhone.val().trim()
    var prefix = '账号'
    if (phone === 'administrator') return true // 特殊账号
    var result = isEmpty(phone, prefix)
    if (!result.status) {
      showFrontendErr(loginPhone, result.text)
      return false
    }
    return true
  }
  // 校验密码
  function checkPassword () {
    var password = loginPassword.val().trim()
    var prefix = '密码'
    var result = isEmpty(password, prefix)
    if (!result.status) {
      showFrontendErr(loginPassword, result.text)
      return false
    }
    return true
  }

  // 提示后端的报错信息
  function showBackendErr (errText) {
    if (errText == undefined || errText.trim() == '') return
    backendErrText.text(errText)
    backendErr.removeClass('hidden')
    backendErr.css({
      top: title.offset().top + title.outerHeight() + 3 + 'px',
      opacity: 1
    })
    clearTimeout(errTimer)
    errTimer = setTimeout(function () {
      backendErr.removeAttr('style')
      backendErr.addClass('hidden')
    }, 8000)
  }

  // 提示前端校验不过的错误信息
  function showFrontendErr (target, errText) {
    var parent = target.parent()
    parent.addClass('input-err')
    var parentOffset = parent.offset()
    var parentOuterHeight = parent.outerHeight()
    frontendErr.css({
      top: parentOffset.top + parentOuterHeight + 2 + 'px',
      left: parentOffset.left + 'px'
    })
    frontendErr.text(errText)
    frontendErr.removeClass('hidden')
    target.focus()
    target.keydown(function () {
      parent.removeClass('input-err')
      frontendErr.addClass('hidden')
      target.off('keydown')
    })
  }

  function randomWord (randomIndicator, min, max) {
    var str = ''
    var range = min
    var arr = ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z']
    if (randomIndicator) {
      range = Math.round(Math.random() * (max - min)) + min
    }
    for (var i = 0; i < range; i++) {
      var pos = Math.round(Math.random() * (arr.length - 1))
      str += arr[pos]
    }
    return str
  }

  function isEmpty (value, textPrefix) {
    if (textPrefix === undefined) textPrefix = ''
    var commonText = '不能为空！'
    if (!value) {
      return { status: false, text: textPrefix + commonText }
    } else {
      return { status: true }
    }
  }
})
